*DECK SNFT1P
      SUBROUTINE SNFT1P(NREG,NMAT,ISCHM,IELEM,NLF,NSCT,U,W,PL,MAT,VOL,
     1 TOTAL,NCODE,ZCODE,QEXT,LFIXUP,FLUX)
*
*-----------------------------------------------------------------------
*
*Purpose:
* perform one inner iteration for solving SN equations in 1D slab
* geometry. Albedo boundary conditions.
*
*Copyright:
* Copyright (C) 2005 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert and A. A. Calloo
*
*Parameters: input
* NREG    number of regions.
* NMAT    number of material mixtures.
* ISCHM   method of spatial discretisation:
*         =1: High-Order Diamond Differencing (HODD) - default;
*         =2: Discontinuous Galerkin finite element method (DG).
* IELEM   measure of order of the spatial approximation polynomial:
*         =1: constant - only for HODD, classical diamond scheme 
*             - default for HODD;
*         =2: linear - default for DG;
*         =3: parabolic;
*         =4: cubic - only for DG.
* NLF     number of $\mu$ levels.
* NSCT    number of Legendre components in the flux.
*         =1: isotropic sources;
*         =2: linearly anisotropic sources.
* U       base points in $\mu$ of the SN quadrature.
* W       weights of the SN quadrature.
* PL      discrete values of the Legendre polynomials corresponding
*         to the SN quadrature.
* MAT     material mixture index in each region.
* VOL     volumes of each region.
* TOTAL   macroscopic total cross sections.
* NCODE   boundary condition indices.
* ZCODE   albedos.
* QEXT    Legendre components of the fixed source.
* LFIXUP  flag to enable negative flux fixup.
*
*Parameters: output
* FLUX    Legendre components of the flux.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER NREG,NMAT,ISCHM,IELEM,NLF,NSCT,MAT(NREG),NCODE(2)
      REAL U(NLF),W(NLF),PL(NSCT,NLF),VOL(NREG),TOTAL(0:NMAT),ZCODE(2),
     1 QEXT(IELEM,NSCT,NREG),FLUX(IELEM,NSCT,NREG)
      LOGICAL LFIXUP
*----
*  LOCAL VARIABLES
*----
      DOUBLE PRECISION AFB,AFB1,AFB2,CURR(2)
      DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:) :: Q,CORNERQ
      DOUBLE PRECISION, ALLOCATABLE, DIMENSION(:,:) :: Q2
      PARAMETER(RLOG=1.0E-8)
*----
*  ALLOCATABLE ARRAYS
*----
      ALLOCATE(Q(IELEM),Q2(IELEM,IELEM+1))
*----
*  PARAMETER VALIDATION.
*----
      IF(ISCHM.EQ.1) THEN 
         IF((IELEM.LT.1).OR.(IELEM.GT.3))
     1      CALL XABORT('SNFT1P: INVALID IELEM (DIAM) VALUE. ' 
     2      //'CHECK INPUT DATA FILE.')
      ELSEIF(ISCHM.EQ.2) THEN
         IF((IELEM.LT.2).OR.(IELEM.GT.4)) 
     1      CALL XABORT('SNFT1P: INVALID IELEM (DIAM) VALUE. '
     2      //'CHECK INPUT DATA FILE.(2)')
      ENDIF
*----
*  COMPUTE A NORMALIZATION CONSTANT.
*----
      DENOM=0.0
      DO 10 I=1+NLF/2,NLF
   10 DENOM=DENOM+W(I)*U(I)
*----
*  OUTER LOOP OVER MU LEVELS.
*----
      CALL XDRSET(FLUX,IELEM*NSCT*NREG,0.0)
      NSCT0=NSCT
      CURR(1)=0.0D0
      CURR(2)=0.0D0
      DO 200 IP=1,NLF/2
      AFB=0.0
      AFB1=0.0
      AFB2=0.0
*----
*  HIGH-ORDER DIAMOND DIFFERENCING METHOD
*----
      IF(ISCHM.EQ.1) THEN 
*----
*  SHOOTING METHOD FOR NON-VACUUM RIGHT BOUNDARY CONDITION.
*----
      IF(ZCODE(2).NE.0.0) THEN
*----
*  BACKWARD SWEEP (FROM RIGHT TO LEFT).
*----
         DO 30 I=NREG,1,-1
            IBM=MAT(I)
            DO 20 IEL=1,IELEM
            Q(IEL)=0.0
            DO 20 IL=0,NSCT-1
   20       Q(IEL)=Q(IEL)+QEXT(IEL,IL+1,I)*PL(IL+1,IP)/2.0
            IF(IELEM.EQ.1) THEN
               Q2(1,2)=Q(1)*VOL(I)-2.0D0*U(IP)*AFB
               Q2(1,1)=TOTAL(IBM)*VOL(I)-2.0D0*U(IP)
            ELSE IF(IELEM.EQ.2) THEN
               Q2(1,1)=TOTAL(IBM)*VOL(I)
               Q2(1,2)=2.0D0*SQRT(3.0D0)*U(IP)
               Q2(1,3)=Q(1)*VOL(I)
               Q2(2,1)=Q2(1,2)
               Q2(2,2)=-TOTAL(IBM)*VOL(I)+6.0D0*U(IP)
               Q2(2,3)=-Q(2)*VOL(I)+2.0D0*SQRT(3.0D0)*U(IP)*AFB
            ELSE IF(IELEM.EQ.3) THEN
               Q2(1,1)=TOTAL(IBM)*VOL(I)-2.0D0*U(IP)
               Q2(1,2)=0.0
               Q2(1,3)=-2.0D0*SQRT(5.0D0)*U(IP)
               Q2(1,4)=Q(1)*VOL(I)-2.0D0*U(IP)*AFB
               Q2(2,1)=Q2(1,2)
               Q2(2,2)=-TOTAL(IBM)*VOL(I)
               Q2(2,3)=-2.0D0*SQRT(15.0D0)*U(IP)
               Q2(2,4)=-Q(2)*VOL(I)
               Q2(3,1)=Q2(1,3)
               Q2(3,2)=Q2(2,3)
               Q2(3,3)=TOTAL(IBM)*VOL(I)-10.0D0*U(IP)
               Q2(3,4)=Q(3)*VOL(I)-2.0D0*SQRT(5.0D0)*U(IP)*AFB
            ENDIF
            CALL ALSBD(IELEM,1,Q2,IER,IELEM)
            IF(IER.NE.0) CALL XABORT('SNFT1P: SINGULAR MATRIX(1).')
            IF(IELEM.EQ.1) THEN
               IF(LFIXUP.AND.(Q2(1,2).LE.RLOG)) Q2(1,2)=0.0
               AFB=2.0D0*Q2(1,2)-AFB
            ELSE IF(IELEM.EQ.2) THEN
               AFB=AFB-2.0D0*SQRT(3.0D0)*Q2(2,3)
            ELSE IF(IELEM.EQ.3) THEN
               AFB=2.0D0*Q2(1,4)+2.0D0*SQRT(5.0D0)*Q2(3,4)-AFB
            ENDIF
            IF(LFIXUP.AND.(AFB.LE.RLOG)) AFB=0.0
   30    CONTINUE
*----
*  FORWARD SWEEP (FROM LEFT TO RIGHT).
*----
         AFBA1=0.0
         IF(NCODE(1).EQ.4) THEN
            AFBA1=REAL(AFB)
            AFB=0.0
         ELSE
            AFB=ZCODE(1)*REAL(AFB)
         ENDIF
         DO 50 I=1,NREG
            IBM=MAT(I)
            DO 40 IEL=1,IELEM
            Q(IEL)=0.0
            DO 40 IL=0,NSCT-1
   40       Q(IEL)=Q(IEL)+QEXT(IEL,IL+1,I)*PL(IL+1,NLF-IP+1)/2.0
            IF(IELEM.EQ.1) THEN
               Q2(1,2)=Q(1)*VOL(I)+2.0D0*U(NLF-IP+1)*AFB
               Q2(1,1)=TOTAL(IBM)*VOL(I)+2.0D0*U(NLF-IP+1)
            ELSE IF(IELEM.EQ.2) THEN
               Q2(1,1)=TOTAL(IBM)*VOL(I)
               Q2(1,2)=2.0D0*SQRT(3.0D0)*U(NLF-IP+1)
               Q2(1,3)=Q(1)*VOL(I)
               Q2(2,1)=Q2(1,2)
               Q2(2,2)=-TOTAL(IBM)*VOL(I)-6.0D0*U(NLF-IP+1)
               Q2(2,3)=-Q(2)*VOL(I)+2.0D0*SQRT(3.0D0)*U(NLF-IP+1)*AFB
            ELSE IF(IELEM.EQ.3) THEN
               Q2(1,1)=TOTAL(IBM)*VOL(I)+2.0D0*U(NLF-IP+1)
               Q2(1,2)=0.0
               Q2(1,3)=2.0D0*SQRT(5.0D0)*U(NLF-IP+1)
               Q2(1,4)=Q(1)*VOL(I)+2.0D0*U(NLF-IP+1)*AFB
               Q2(2,1)=Q2(1,2)
               Q2(2,2)=-TOTAL(IBM)*VOL(I)
               Q2(2,3)=-2.0D0*SQRT(15.0D0)*U(NLF-IP+1)
               Q2(2,4)=-Q(2)*VOL(I)
               Q2(3,1)=Q2(1,3)
               Q2(3,2)=Q2(2,3)
               Q2(3,3)=TOTAL(IBM)*VOL(I)+10.0D0*U(NLF-IP+1)
               Q2(3,4)=Q(3)*VOL(I)+2.0D0*SQRT(5.0D0)*U(NLF-IP+1)*AFB
            ENDIF
            CALL ALSBD(IELEM,1,Q2,IER,IELEM)
            IF(IER.NE.0) CALL XABORT('SNFT1P: SINGULAR MATRIX(2).')
            IF(IELEM.EQ.1) THEN
               IF(LFIXUP.AND.(Q2(1,2).LE.RLOG)) Q2(1,2)=0.0
               AFB=2.0D0*Q2(1,2)-AFB
            ELSE IF(IELEM.EQ.2) THEN
               AFB=AFB+2.0D0*SQRT(3.0D0)*Q2(2,3)
            ELSE IF(IELEM.EQ.3) THEN
               AFB=2.0D0*Q2(1,4)+2.0D0*SQRT(5.0D0)*Q2(3,4)-AFB
            ENDIF
            IF(LFIXUP.AND.(AFB.LE.RLOG)) AFB=0.0
   50    CONTINUE
*----
*  BACKWARD SWEEP (FROM RIGHT TO LEFT).
*----
         AFBA2=0.0
         AFBA=0.0
         IF(NCODE(1).EQ.4) THEN
            AFBA2=REAL(AFB)
         ELSE
            AFBA=REAL(AFB)
         ENDIF
         AFB=1.0
         DO 70 I=NREG,1,-1
            IBM=MAT(I)
            DO 60 IEL=1,IELEM
            Q(IEL)=0.0
            DO 60 IL=0,NSCT-1
   60       Q(IEL)=Q(IEL)+QEXT(IEL,IL+1,I)*PL(IL+1,IP)/2.0
            IF(IELEM.EQ.1) THEN
               Q2(1,2)=Q(1)*VOL(I)-2.0D0*U(IP)*AFB
               Q2(1,1)=TOTAL(IBM)*VOL(I)-2.0D0*U(IP)
            ELSE IF(IELEM.EQ.2) THEN
               Q2(1,1)=TOTAL(IBM)*VOL(I)
               Q2(1,2)=2.0D0*SQRT(3.0D0)*U(IP)
               Q2(1,3)=Q(1)*VOL(I)
               Q2(2,1)=Q2(1,2)
               Q2(2,2)=-TOTAL(IBM)*VOL(I)+6.0D0*U(IP)
               Q2(2,3)=-Q(2)*VOL(I)+2.0D0*SQRT(3.0D0)*U(IP)*AFB
            ELSE IF(IELEM.EQ.3) THEN
               Q2(1,1)=TOTAL(IBM)*VOL(I)-2.0D0*U(IP)
               Q2(1,2)=0.0
               Q2(1,3)=-2.0D0*SQRT(5.0D0)*U(IP)
               Q2(1,4)=Q(1)*VOL(I)-2.0D0*U(IP)*AFB
               Q2(2,1)=Q2(1,2)
               Q2(2,2)=-TOTAL(IBM)*VOL(I)
               Q2(2,3)=-2.0D0*SQRT(15.0D0)*U(IP)
               Q2(2,4)=-Q(2)*VOL(I)
               Q2(3,1)=Q2(1,3)
               Q2(3,2)=Q2(2,3)
               Q2(3,3)=TOTAL(IBM)*VOL(I)-10.0D0*U(IP)
               Q2(3,4)=Q(3)*VOL(I)-2.0D0*SQRT(5.0D0)*U(IP)*AFB
            ENDIF
            CALL ALSBD(IELEM,1,Q2,IER,IELEM)
            IF(IER.NE.0) CALL XABORT('SNFT1P: SINGULAR MATRIX(3).')
            IF(IELEM.EQ.1) THEN
               IF(LFIXUP.AND.(Q2(1,2).LE.RLOG)) Q2(1,2)=0.0
               AFB=2.0D0*Q2(1,2)-AFB
            ELSE IF(IELEM.EQ.2) THEN
               AFB=AFB-2.0D0*SQRT(3.0D0)*Q2(2,3)
            ELSE IF(IELEM.EQ.3) THEN
               AFB=2.0D0*Q2(1,4)+2.0D0*SQRT(5.0D0)*Q2(3,4)-AFB
            ENDIF
            IF(LFIXUP.AND.(AFB.LE.RLOG)) AFB=0.0
   70    CONTINUE
*----
*  FORWARD SWEEP (FROM LEFT TO RIGHT).
*----
         IF(NCODE(1).EQ.4) THEN
            AFBB1=REAL(AFB)
            AFB1=AFBA1/(1.0+AFBA1-AFBB1)
            AFB=1.0
         ELSE
            AFB=ZCODE(1)*AFB
         ENDIF
         DO 90 I=1,NREG
            IBM=MAT(I)
            DO 80 IEL=1,IELEM
            Q(IEL)=0.0
            DO 80 IL=0,NSCT-1
   80       Q(IEL)=Q(IEL)+QEXT(IEL,IL+1,I)*PL(IL+1,NLF-IP+1)/2.0
            IF(IELEM.EQ.1) THEN
               Q2(1,2)=Q(1)*VOL(I)+2.0D0*U(NLF-IP+1)*AFB
               Q2(1,1)=TOTAL(IBM)*VOL(I)+2.0D0*U(NLF-IP+1)
            ELSE IF(IELEM.EQ.2) THEN
               Q2(1,1)=TOTAL(IBM)*VOL(I)
               Q2(1,2)=2.0D0*SQRT(3.0D0)*U(NLF-IP+1)
               Q2(1,3)=Q(1)*VOL(I)
               Q2(2,1)=Q2(1,2)
               Q2(2,2)=-TOTAL(IBM)*VOL(I)-6.0D0*U(NLF-IP+1)
               Q2(2,3)=-Q(2)*VOL(I)+2.0D0*SQRT(3.0D0)*U(NLF-IP+1)*AFB
            ELSE IF(IELEM.EQ.3) THEN
               Q2(1,1)=TOTAL(IBM)*VOL(I)+2.0D0*U(NLF-IP+1)
               Q2(1,2)=0.0
               Q2(1,3)=2.0D0*SQRT(5.0D0)*U(NLF-IP+1)
               Q2(1,4)=Q(1)*VOL(I)+2.0D0*U(NLF-IP+1)*AFB
               Q2(2,1)=Q2(1,2)
               Q2(2,2)=-TOTAL(IBM)*VOL(I)
               Q2(2,3)=-2.0D0*SQRT(15.0D0)*U(NLF-IP+1)
               Q2(2,4)=-Q(2)*VOL(I)
               Q2(3,1)=Q2(1,3)
               Q2(3,2)=Q2(2,3)
               Q2(3,3)=TOTAL(IBM)*VOL(I)+10.0D0*U(NLF-IP+1)
               Q2(3,4)=Q(3)*VOL(I)+2.0D0*SQRT(5.0D0)*U(NLF-IP+1)*AFB
            ENDIF
            CALL ALSBD(IELEM,1,Q2,IER,IELEM)
            IF(IER.NE.0) CALL XABORT('SNFT1P: SINGULAR MATRIX(4).')
            IF(IELEM.EQ.1) THEN
               IF(LFIXUP.AND.(Q2(1,2).LE.RLOG)) Q2(1,2)=0.0
               AFB=2.0D0*Q2(1,2)-AFB
               IF(LFIXUP.AND.(AFB.LE.RLOG)) AFB=0.0
            ELSE IF(IELEM.EQ.2) THEN
               AFB=AFB+2.0D0*SQRT(3.0D0)*Q2(2,3)
            ELSE IF(IELEM.EQ.3) THEN
               AFB=2.0D0*Q2(1,4)+2.0D0*SQRT(5.0D0)*Q2(3,4)-AFB
            ENDIF
   90    CONTINUE
         IF(NCODE(1).EQ.4) THEN
            AFBB2=REAL(AFB)
            AFB2=AFBA2/(1.0+AFBA2-AFBB2)
         ELSE
            AFBB=REAL(AFB)
            AFB=ZCODE(2)*AFBA/(1.0+ZCODE(2)*(AFBA-AFBB))
         ENDIF
      ENDIF
*----
*  BACKWARD SWEEP (FROM RIGHT TO LEFT).
*----
      IF(NCODE(1).EQ.4) AFB=AFB1
      DO 120 I=NREG,1,-1
         IBM=MAT(I)
         DO 100 IEL=1,IELEM
         Q(IEL)=0.0
         DO 100 IL=0,NSCT-1
  100    Q(IEL)=Q(IEL)+QEXT(IEL,IL+1,I)*PL(IL+1,IP)/2.0
         IF(IELEM.EQ.1) THEN
            Q2(1,2)=Q(1)*VOL(I)-2.0D0*U(IP)*AFB
            Q2(1,1)=TOTAL(IBM)*VOL(I)-2.0D0*U(IP)
         ELSE IF(IELEM.EQ.2) THEN
            Q2(1,1)=TOTAL(IBM)*VOL(I)
            Q2(1,2)=2.0D0*SQRT(3.0D0)*U(IP)
            Q2(1,3)=Q(1)*VOL(I)
            Q2(2,1)=Q2(1,2)
            Q2(2,2)=-TOTAL(IBM)*VOL(I)+6.0D0*U(IP)
            Q2(2,3)=-Q(2)*VOL(I)+2.0D0*SQRT(3.0D0)*U(IP)*AFB
         ELSE IF(IELEM.EQ.3) THEN
            Q2(1,1)=TOTAL(IBM)*VOL(I)-2.0D0*U(IP)
            Q2(1,2)=0.0
            Q2(1,3)=-2.0D0*SQRT(5.0D0)*U(IP)
            Q2(1,4)=Q(1)*VOL(I)-2.0D0*U(IP)*AFB
            Q2(2,1)=Q2(1,2)
            Q2(2,2)=-TOTAL(IBM)*VOL(I)
            Q2(2,3)=-2.0D0*SQRT(15.0D0)*U(IP)
            Q2(2,4)=-Q(2)*VOL(I)
            Q2(3,1)=Q2(1,3)
            Q2(3,2)=Q2(2,3)
            Q2(3,3)=TOTAL(IBM)*VOL(I)-10.0D0*U(IP)
            Q2(3,4)=Q(3)*VOL(I)-2.0D0*SQRT(5.0D0)*U(IP)*AFB
         ENDIF
         CALL ALSBD(IELEM,1,Q2,IER,IELEM)
         IF(IER.NE.0) CALL XABORT('SNFT1P: SINGULAR MATRIX(5).')
         IF(IELEM.EQ.1) THEN
            IF(LFIXUP.AND.(Q2(1,2).LE.RLOG)) Q2(1,2)=0.0
            AFB=2.0D0*Q2(1,2)-AFB
         ELSE IF(IELEM.EQ.2) THEN
            AFB=AFB-2.0D0*SQRT(3.0D0)*Q2(2,3)
         ELSE IF(IELEM.EQ.3) THEN
            AFB=2.0D0*Q2(1,4)+2.0D0*SQRT(5.0D0)*Q2(3,4)-AFB
         ENDIF
         IF(LFIXUP.AND.(AFB.LE.RLOG)) AFB=0.0
         DO 110 K=1,NSCT
         DO 110 IEL=1,IELEM
  110    FLUX(IEL,K,I)=FLUX(IEL,K,I)+W(IP)*REAL(Q2(IEL,IELEM+1))*
     1   PL(K,IP)
  120 CONTINUE
      CURR(1)=CURR(1)+W(IP)*U(IP)*AFB
*----
*  FORWARD SWEEP (FROM LEFT TO RIGHT).
*----
      AFB=ZCODE(1)*AFB
      IF(NCODE(1).EQ.4) AFB=AFB2
      DO 150 I=1,NREG
         IBM=MAT(I)
         DO 130 IEL=1,IELEM
         Q(IEL)=0.0
         DO 130 IL=0,NSCT-1
  130    Q(IEL)=Q(IEL)+QEXT(IEL,IL+1,I)*PL(IL+1,NLF-IP+1)/2.0
         IF(IELEM.EQ.1) THEN
            Q2(1,2)=Q(1)*VOL(I)+2.0D0*U(NLF-IP+1)*AFB
            Q2(1,1)=TOTAL(IBM)*VOL(I)+2.0D0*U(NLF-IP+1)
         ELSE IF(IELEM.EQ.2) THEN
            Q2(1,1)=TOTAL(IBM)*VOL(I)
            Q2(1,2)=2.0D0*SQRT(3.0D0)*U(NLF-IP+1)
            Q2(1,3)=Q(1)*VOL(I)
            Q2(2,1)=Q2(1,2)
            Q2(2,2)=-TOTAL(IBM)*VOL(I)-6.0D0*U(NLF-IP+1)
            Q2(2,3)=-Q(2)*VOL(I)+2.0D0*SQRT(3.0D0)*U(NLF-IP+1)*AFB
         ELSE IF(IELEM.EQ.3) THEN
            Q2(1,1)=TOTAL(IBM)*VOL(I)+2.0D0*U(NLF-IP+1)
            Q2(1,2)=0.0
            Q2(1,3)=2.0D0*SQRT(5.0D0)*U(NLF-IP+1)
            Q2(1,4)=Q(1)*VOL(I)+2.0D0*U(NLF-IP+1)*AFB
            Q2(2,1)=Q2(1,2)
            Q2(2,2)=-TOTAL(IBM)*VOL(I)
            Q2(2,3)=-2.0D0*SQRT(15.0D0)*U(NLF-IP+1)
            Q2(2,4)=-Q(2)*VOL(I)
            Q2(3,1)=Q2(1,3)
            Q2(3,2)=Q2(2,3)
            Q2(3,3)=TOTAL(IBM)*VOL(I)+10.0D0*U(NLF-IP+1)
            Q2(3,4)=Q(3)*VOL(I)+2.0D0*SQRT(5.0D0)*U(NLF-IP+1)*AFB
         ENDIF
         CALL ALSBD(IELEM,1,Q2,IER,IELEM)
         IF(IER.NE.0) CALL XABORT('SNFT1P: SINGULAR MATRIX(6).')
         IF(IELEM.EQ.1) THEN
            IF(LFIXUP.AND.(Q2(1,2).LE.RLOG)) Q2(1,2)=0.0
            AFB=2.0D0*Q2(1,2)-AFB
         ELSE IF(IELEM.EQ.2) THEN
            AFB=AFB+2.0D0*SQRT(3.0D0)*Q2(2,3)
         ELSE IF(IELEM.EQ.3) THEN
            AFB=2.0D0*Q2(1,4)+2.0D0*SQRT(5.0D0)*Q2(3,4)-AFB
         ENDIF
         IF(LFIXUP.AND.(AFB.LE.RLOG)) AFB=0.0
         DO 140 K=1,NSCT
         DO 140 IEL=1,IELEM
  140    FLUX(IEL,K,I)=FLUX(IEL,K,I)+W(NLF-IP+1)*REAL(Q2(IEL,IELEM+1))*
     1   PL(K,NLF-IP+1)
  150 CONTINUE
      CURR(2)=CURR(2)+W(NLF-IP+1)*U(NLF-IP+1)*AFB
*----
*  DISCONTINUOUS GALERKIN FINITE ELEMENT METHOD
*----
      ELSEIF(ISCHM.EQ.2) THEN
         ALLOCATE(CORNERQ(2))
*----
*  SHOOTING METHOD FOR NON-VACUUM RIGHT BOUNDARY CONDITION.
*----
      IF(ZCODE(2).NE.0.0) THEN
*----
*  BACKWARD SWEEP (FROM RIGHT TO LEFT).
*----
         DO 930 I=NREG,1,-1 
            CORNERQ = 0.0
            IBM=MAT(I)
  
            DO IEL=1,IELEM
               Q(IEL)=0.0
               DO IL=0,NSCT-1
                  Q(IEL)=Q(IEL) + QEXT(IEL,IL+1,I)*PL(IL+1,IP)/2.0
               ENDDO
            ENDDO 

            IF(IELEM.EQ.2) THEN
            Q2(1,1) = U(IP)*(-1.0/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/2.0) 
            Q2(2,1) = U(IP)*(-SQRT(3.0D0)/6.0) + 
     >         (TOTAL(IBM)*VOL(I))*(0.0) 
            Q2(1,2) = U(IP)*(SQRT(3.0D0)/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(0.0) 
            Q2(2,2) = U(IP)*(-1.0/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/6.0) 
            Q2(1,3) = (Q(1)*(1.0/2.0))*VOL(I)
     >         + U(IP)*AFB*(-1.0/2.0)
            Q2(2,3) = (Q(2)*(1.0/6.0))*VOL(I)
     >         + U(IP)*AFB*(-1.0*SQRT(3.0D0)/6.0)
            ELSE IF(IELEM.EQ.3) THEN
            Q2(1,1) = U(IP)*(-1.0/6.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/2.0) 
            Q2(2,1) = U(IP)*(-SQRT(3.0)/6.0) + 
     >         (TOTAL(IBM)*VOL(I))*(0.0) 
            Q2(3,1) = U(IP)*(-SQRT(5.0)/15.0) + 
     >         (TOTAL(IBM)*VOL(I))*(-SQRT(5.0)/15.0)

            Q2(1,2) = U(IP)*(5.0*SQRT(3.0)/6.0) + 
     >        (TOTAL(IBM)*VOL(I))*(0.0)
            Q2(2,2) = U(IP)*(-1.0/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/6.0) 
            Q2(3,2) = U(IP)*(-SQRT(15.0)/5.0) + 
     >        (TOTAL(IBM)*VOL(I))*(0.0)

            Q2(1,3) = U(IP)*(-5.0*SQRT(5.0)/30.0) + 
     >        (TOTAL(IBM)*VOL(I))*(-2.0*SQRT(5.0)/30.0)
            Q2(2,3) = U(IP)*(SQRT(15.0)/6.0) + 
     >        (TOTAL(IBM)*VOL(I))*(0.0)
            Q2(3,3) = U(IP)*(-1.0/3.0) + 
     >        (TOTAL(IBM)*VOL(I))*(2.0/15.0)

            Q2(1,4) = (Q(1)*(90.0/180.0)
     >         + Q(3)*(-12.0*SQRT(5.0)/180.0))*VOL(I)
     >         + U(IP)*AFB*(-1.0/6.0)
            Q2(2,4) = (Q(2)*(1.0/6.0))*VOL(I)
     >         + U(IP)*AFB*(-1.0*SQRT(3.0)/6.0)
            Q2(3,4) = (Q(1)*(-30.0*SQRT(5.0)/450.0) 
     >         + Q(3)*(12.0*(5.0)/450.0))*VOL(I)
     >         + U(IP)*AFB*(-1.0*SQRT(5.0)/15.0)
            ELSE IF(IELEM.EQ.4) THEN
            Q2(1,1) = (5*TOTAL(IBM)*VOL(I))/16 - U(IP)/8 
            Q2(1,2) = (3**(0.5D0)*U(IP))/2 
            Q2(1,3) = -(5**(0.5D0)*(10*U(IP) + 
     >         3*TOTAL(IBM)*VOL(I)))/80
            Q2(1,4) = (7**(0.5D0)*U(IP))/8 
            
            Q2(2,1) = -(11*3**(0.5D0)*U(IP))/120 
            Q2(2,2) = (17*TOTAL(IBM)*VOL(I))/48 - (11*U(IP))/40 
            Q2(2,3) = (37*15**(0.5D0)*U(IP))/60 
            Q2(2,4) = -(3**(0.5D0)*7**(0.5D0)*(22*U(IP) + 
     >         9*TOTAL(IBM)*VOL(I)))/240 
            
            Q2(3,1) = -(3*5**(0.5D0)*(2*U(IP) + 
     >         TOTAL(IBM)*VOL(I)))/80 
            Q2(3,2) = -(3*15**(0.5D0)*U(IP))/20 
            Q2(3,3) = (9*TOTAL(IBM)*VOL(I))/80 - (3*U(IP))/8 
            Q2(3,4) = (3*35**(0.5D0)*U(IP))/40 
            
            Q2(4,1) = -(9*7**(0.5D0)*U(IP))/280 
            Q2(4,2) = -(3*3**(0.5D0)*7**(0.5D0)*(6*U(IP) + 
     >         7*TOTAL(IBM)*VOL(I)))/560 
            Q2(4,3) = -(9*35**(0.5D0)*U(IP))/35 
            Q2(4,4) = (81*TOTAL(IBM)*VOL(I))/560 - (9*U(IP))/40 
            
            Q2(1,5) = (((50*Q(01) - 
     >         6*5**(0.5D0)*Q(03)))/160)*VOL(I) 
            Q2(2,5) = (((85*Q(02) - 
     >         9*21**(0.5D0)*Q(04)))/240)*VOL(I) 
            Q2(3,5) = ((3*(3*Q(03) - 
     >         5**(0.5D0)*Q(01)))/80)*VOL(I) 
            Q2(4,5) = ((3*(27*Q(04) - 
     >         7*21**(0.5D0)*Q(02)))/560)*VOL(I) 
            
            Q2(1,5) = Q2(1,5) + (-AFB/8)*U(IP) 
            Q2(2,5) = Q2(2,5) + (-(11*3**(0.5D0)*AFB)/120)*U(IP) 
            Q2(3,5) = Q2(3,5) + (-(3*5**(0.5D0)*AFB)/40)*U(IP) 
            Q2(4,5) = Q2(4,5) + (-(9*7**(0.5D0)*AFB)/280)*U(IP) 
            ENDIF

            CALL ALSBD(IELEM,1,Q2,IER,IELEM)
            IF(IER.NE.0) CALL XABORT('SNFT1P: SINGULAR MATRIX(1).')

            IF(IELEM.EQ.2)THEN
            CORNERQ(1) = Q2(1,IELEM+1) +
     >         Q2(2,IELEM+1)*(-SQRT(3.0D0)) 
            CORNERQ(2) = Q2(1,IELEM+1) + 
     >         Q2(2,IELEM+1)*(SQRT(3.0D0))
            ELSEIF(IELEM.EQ.3)THEN
            CORNERQ(1) = Q2(1,IELEM+1) + 
     >         Q2(2,IELEM+1)*(-SQRT(3.0)) + 
     >         Q2(3,IELEM+1)*(SQRT(5.0)) 
            CORNERQ(2) = Q2(1,IELEM+1) + 
     >         Q2(2,IELEM+1)*(SQRT(3.0)) + 
     >         Q2(3,IELEM+1)*(SQRT(5.0)) 
            ELSEIF(IELEM.EQ.4)THEN
            CORNERQ(1) = (Q2(01,IELEM+1) - 3**(0.5D0)*Q2(02,IELEM+1) 
     >         + 5**(0.5D0)*Q2(03,IELEM+1) - 7**(0.5D0)*Q2(04,IELEM+1)) 
            CORNERQ(2) = (Q2(01,IELEM+1) + 3**(0.5D0)*Q2(02,IELEM+1) 
     >         + 5**(0.5D0)*Q2(03,IELEM+1) + 7**(0.5D0)*Q2(04,IELEM+1)) 
            ENDIF
            AFB=CORNERQ(1)
  930    CONTINUE
*----
*  FORWARD SWEEP (FROM LEFT TO RIGHT).
*----
         AFBA1=0.0
         IF(NCODE(1).EQ.4) THEN
            AFBA1=REAL(AFB)
            AFB=0.0
         ELSE
            AFB=ZCODE(1)*REAL(AFB)
         ENDIF
         DO 950 I=1,NREG
            CORNERQ = 0.0
            IBM=MAT(I)

            DO IEL=1,IELEM
               Q(IEL)=0.0
               DO IL=0,NSCT-1
                  Q(IEL)=Q(IEL) + 
     1               QEXT(IEL,IL+1,I)*PL(IL+1,NLF-IP+1)/2.0
               END DO
            END DO

            IF(IELEM.EQ.2) THEN
            Q2(1,1) = U(NLF-IP+1)*(1.0/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/2.0) 
            Q2(2,1) = U(NLF-IP+1)*(-SQRT(3.0)/6.0) + 
     >         (TOTAL(IBM)*VOL(I))*(0.0) 
            Q2(1,2) = U(NLF-IP+1)*(SQRT(3.0)/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(0.0) 
            Q2(2,2) = U(NLF-IP+1)*(1.0/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/6.0) 
            Q2(1,3) = (Q(1)*(1.0/2.0))*VOL(I)
     >         + U(NLF-IP+1)*AFB*(1.0/2.0)
            Q2(2,3) = (Q(2)*(1.0/6.0))*VOL(I)
     >         + U(NLF-IP+1)*AFB*(-1.0*SQRT(3.0)/6.0)
            ELSE IF(IELEM.EQ.3) THEN
            Q2(1,1) = U(NLF-IP+1)*(1.0/6.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/2.0) 
            Q2(2,1) = U(NLF-IP+1)*(-SQRT(3.0)/6.0) + 
     >         (TOTAL(IBM)*VOL(I))*(0.0) 
            Q2(3,1) = U(NLF-IP+1)*(SQRT(5.0)/15.0) + 
     >         (TOTAL(IBM)*VOL(I))*(-SQRT(5.0)/15.0)

            Q2(1,2) = U(NLF-IP+1)*(5.0*SQRT(3.0)/6.0) + 
     >        (TOTAL(IBM)*VOL(I))*(0.0)
            Q2(2,2) = U(NLF-IP+1)*(1.0/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/6.0) 
            Q2(3,2) = U(NLF-IP+1)*(-SQRT(15.0)/5.0) + 
     >        (TOTAL(IBM)*VOL(I))*(0.0)

            Q2(1,3) = U(NLF-IP+1)*(5.0*SQRT(5.0)/30.0) + 
     >        (TOTAL(IBM)*VOL(I))*(-2.0*SQRT(5.0)/30.0)
            Q2(2,3) = U(NLF-IP+1)*(SQRT(15.0)/6.0) + 
     >        (TOTAL(IBM)*VOL(I))*(0.0)
            Q2(3,3) = U(NLF-IP+1)*(1.0/3.0) + 
     >        (TOTAL(IBM)*VOL(I))*(2.0/15.0)

            Q2(1,4) = (Q(1)*(90.0/180.0)
     >         + Q(3)*(-12.0*SQRT(5.0)/180.0))*VOL(I)
     >         + U(NLF-IP+1)*AFB*(1.0/6.0)
            Q2(2,4) = (Q(2)*(1.0/6.0))*VOL(I)
     >         + U(NLF-IP+1)*AFB*(-1.0*SQRT(3.0)/6.0)
            Q2(3,4) = (Q(1)*(-30.0*SQRT(5.0)/450.0) 
     >         + Q(3)*(12.0*(5.0)/450.0))*VOL(I)
     >         + U(NLF-IP+1)*AFB*(1.0*SQRT(5.0)/15.0)
            ELSE IF(IELEM.EQ.4) THEN
            Q2(1,1) = U(NLF-IP+1)/8 + (5*TOTAL(IBM)*VOL(I))/16 
            Q2(1,2) = (3**(0.5D0)*U(NLF-IP+1))/2 
            Q2(1,3) = (5**(0.5D0)*(10*U(NLF-IP+1) - 
     >         3*TOTAL(IBM)*VOL(I)))/80 
            Q2(1,4) = (7**(0.5D0)*U(NLF-IP+1))/8 

            Q2(2,1) = -(11*3**(0.5D0)*U(NLF-IP+1))/120 
            Q2(2,2) = (11*U(NLF-IP+1))/40 + 
     >         (17*TOTAL(IBM)*VOL(I))/48 
            Q2(2,3) = (37*15**(0.5D0)*U(NLF-IP+1))/60 
            Q2(2,4) = (3**(0.5D0)*7**(0.5D0)*(22*U(NLF-IP+1) - 
     >         9*TOTAL(IBM)*VOL(I)))/240 

            Q2(3,1) = (3*5**(0.5D0)*(2*U(NLF-IP+1) - 
     >         TOTAL(IBM)*VOL(I)))/80 
            Q2(3,2) = -(3*15**(0.5D0)*U(NLF-IP+1))/20 
            Q2(3,3) = (3*U(NLF-IP+1))/8 + (9*TOTAL(IBM)*VOL(I))/80 
            Q2(3,4) = (3*35**(0.5D0)*U(NLF-IP+1))/40 

            Q2(4,1) = -(9*7**(0.5D0)*U(NLF-IP+1))/280 
            Q2(4,2) = (3*3**(0.5D0)*7**(0.5D0)*(6*U(NLF-IP+1) - 
     >         7*TOTAL(IBM)*VOL(I)))/560 
            Q2(4,3) = -(9*35**(0.5D0)*U(NLF-IP+1))/35 
            Q2(4,4) = (9*U(NLF-IP+1))/40 + 
     >         (81*TOTAL(IBM)*VOL(I))/560 

            Q2(1,5) = (((50*Q(01) - 
     >         6*5**(0.5D0)*Q(03)))/160)*VOL(I) 
            Q2(2,5) = (((85*Q(02) - 
     >         9*21**(0.5D0)*Q(04)))/240)*VOL(I) 
            Q2(3,5) = ((3*(3*Q(03) - 
     >         5**(0.5D0)*Q(01)))/80)*VOL(I) 
            Q2(4,5) = ((3*(27*Q(04) - 
     >         7*21**(0.5D0)*Q(02)))/560)*VOL(I) 

            Q2(1,5) = Q2(1,5) + (AFB/8)*U(NLF-IP+1) 
            Q2(2,5) = Q2(2,5) + (-(11*3**(0.5D0)*AFB)/120)*U(NLF-IP+1) 
            Q2(3,5) = Q2(3,5) + ((3*5**(0.5D0)*AFB)/40)*U(NLF-IP+1) 
            Q2(4,5) = Q2(4,5) + (-(9*7**(0.5D0)*AFB)/280)*U(NLF-IP+1)
            ENDIF

            CALL ALSBD(IELEM,1,Q2,IER,IELEM)
            IF(IER.NE.0) CALL XABORT('SNFT1P: SINGULAR MATRIX(2).')
            
            IF(IELEM.EQ.2)THEN
            CORNERQ(1) = Q2(1,IELEM+1) +
     >         Q2(2,IELEM+1)*(-SQRT(3.0)) 
            CORNERQ(2) = Q2(1,IELEM+1) + 
     >         Q2(2,IELEM+1)*(SQRT(3.0))
            ELSEIF(IELEM.EQ.3)THEN
            CORNERQ(1) = Q2(1,IELEM+1) + 
     >         Q2(2,IELEM+1)*(-SQRT(3.0)) + 
     >         Q2(3,IELEM+1)*(SQRT(5.0)) 
            CORNERQ(2) = Q2(1,IELEM+1) + 
     >         Q2(2,IELEM+1)*(SQRT(3.0)) + 
     >         Q2(3,IELEM+1)*(SQRT(5.0)) 
            ELSEIF(IELEM.EQ.4)THEN
            CORNERQ(1) = (Q2(01,IELEM+1) - 3**(0.5D0)*Q2(02,IELEM+1) 
     >         + 5**(0.5D0)*Q2(03,IELEM+1) - 7**(0.5D0)*Q2(04,IELEM+1)) 
            CORNERQ(2) = (Q2(01,IELEM+1) + 3**(0.5D0)*Q2(02,IELEM+1) 
     >         + 5**(0.5D0)*Q2(03,IELEM+1) + 7**(0.5D0)*Q2(04,IELEM+1)) 
            ENDIF
            AFB=CORNERQ(2)
  950    CONTINUE
*----
*  BACKWARD SWEEP (FROM RIGHT TO LEFT).
*----
         AFBA2=0.0
         AFBA=0.0
         IF(NCODE(1).EQ.4) THEN
            AFBA2=REAL(AFB)
         ELSE
            AFBA=REAL(AFB)
         ENDIF
         AFB=1.0
         DO 970 I=NREG,1,-1
            CORNERQ = 0.0
            IBM=MAT(I)
            
            DO IEL=1,IELEM
               Q(IEL)=0.0
               DO IL=0,NSCT-1
                  Q(IEL)=Q(IEL)+QEXT(IEL,IL+1,I)*PL(IL+1,IP)/2.0
               END DO
            END DO 

            IF(IELEM.EQ.2) THEN
            Q2(1,1) = U(IP)*(-1.0/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/2.0) 
            Q2(2,1) = U(IP)*(-SQRT(3.0D0)/6.0) + 
     >         (TOTAL(IBM)*VOL(I))*(0.0) 
            Q2(1,2) = U(IP)*(SQRT(3.0D0)/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(0.0) 
            Q2(2,2) = U(IP)*(-1.0/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/6.0) 
            Q2(1,3) = (Q(1)*(1.0/2.0))*VOL(I)
     >         + U(IP)*AFB*(-1.0/2.0)
            Q2(2,3) = (Q(2)*(1.0/6.0))*VOL(I)
     >         + U(IP)*AFB*(-1.0*SQRT(3.0D0)/6.0)
            ELSE IF(IELEM.EQ.3) THEN
            Q2(1,1) = U(IP)*(-1.0/6.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/2.0) 
            Q2(2,1) = U(IP)*(-SQRT(3.0)/6.0) + 
     >         (TOTAL(IBM)*VOL(I))*(0.0) 
            Q2(3,1) = U(IP)*(-SQRT(5.0)/15.0) + 
     >         (TOTAL(IBM)*VOL(I))*(-SQRT(5.0)/15.0)

            Q2(1,2) = U(IP)*(5.0*SQRT(3.0)/6.0) + 
     >        (TOTAL(IBM)*VOL(I))*(0.0)
            Q2(2,2) = U(IP)*(-1.0/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/6.0) 
            Q2(3,2) = U(IP)*(-SQRT(15.0)/5.0) + 
     >        (TOTAL(IBM)*VOL(I))*(0.0)

            Q2(1,3) = U(IP)*(-5.0*SQRT(5.0)/30.0) + 
     >        (TOTAL(IBM)*VOL(I))*(-2.0*SQRT(5.0)/30.0)
            Q2(2,3) = U(IP)*(SQRT(15.0)/6.0) + 
     >        (TOTAL(IBM)*VOL(I))*(0.0)
            Q2(3,3) = U(IP)*(-1.0/3.0) + 
     >        (TOTAL(IBM)*VOL(I))*(2.0/15.0)

            Q2(1,4) = (Q(1)*(90.0/180.0)
     >         + Q(3)*(-12.0*SQRT(5.0)/180.0))*VOL(I)
     >         + U(IP)*AFB*(-1.0/6.0)
            Q2(2,4) = (Q(2)*(1.0/6.0))*VOL(I)
     >         + U(IP)*AFB*(-1.0*SQRT(3.0)/6.0)
            Q2(3,4) = (Q(1)*(-30.0*SQRT(5.0)/450.0) 
     >         + Q(3)*(12.0*(5.0)/450.0))*VOL(I)
     >         + U(IP)*AFB*(-1.0*SQRT(5.0)/15.0)
            ELSE IF(IELEM.EQ.4) THEN
            Q2(1,1) = (5*TOTAL(IBM)*VOL(I))/16 - U(IP)/8 
            Q2(1,2) = (3**(0.5D0)*U(IP))/2 
            Q2(1,3) = -(5**(0.5D0)*(10*U(IP) + 
     >         3*TOTAL(IBM)*VOL(I)))/80
            Q2(1,4) = (7**(0.5D0)*U(IP))/8 
            
            Q2(2,1) = -(11*3**(0.5D0)*U(IP))/120 
            Q2(2,2) = (17*TOTAL(IBM)*VOL(I))/48 - (11*U(IP))/40 
            Q2(2,3) = (37*15**(0.5D0)*U(IP))/60 
            Q2(2,4) = -(3**(0.5D0)*7**(0.5D0)*(22*U(IP) + 
     >         9*TOTAL(IBM)*VOL(I)))/240 
            
            Q2(3,1) = -(3*5**(0.5D0)*(2*U(IP) + 
     >         TOTAL(IBM)*VOL(I)))/80 
            Q2(3,2) = -(3*15**(0.5D0)*U(IP))/20 
            Q2(3,3) = (9*TOTAL(IBM)*VOL(I))/80 - (3*U(IP))/8 
            Q2(3,4) = (3*35**(0.5D0)*U(IP))/40 
            
            Q2(4,1) = -(9*7**(0.5D0)*U(IP))/280 
            Q2(4,2) = -(3*3**(0.5D0)*7**(0.5D0)*(6*U(IP) + 
     >         7*TOTAL(IBM)*VOL(I)))/560 
            Q2(4,3) = -(9*35**(0.5D0)*U(IP))/35 
            Q2(4,4) = (81*TOTAL(IBM)*VOL(I))/560 - (9*U(IP))/40 
            
            Q2(1,5) = (((50*Q(01) - 
     >         6*5**(0.5D0)*Q(03)))/160)*VOL(I) 
            Q2(2,5) = (((85*Q(02) - 
     >         9*21**(0.5D0)*Q(04)))/240)*VOL(I) 
            Q2(3,5) = ((3*(3*Q(03) - 
     >         5**(0.5D0)*Q(01)))/80)*VOL(I) 
            Q2(4,5) = ((3*(27*Q(04) - 
     >         7*21**(0.5D0)*Q(02)))/560)*VOL(I) 
            
            Q2(1,5) = Q2(1,5) + (-AFB/8)*U(IP) 
            Q2(2,5) = Q2(2,5) + (-(11*3**(0.5D0)*AFB)/120)*U(IP) 
            Q2(3,5) = Q2(3,5) + (-(3*5**(0.5D0)*AFB)/40)*U(IP) 
            Q2(4,5) = Q2(4,5) + (-(9*7**(0.5D0)*AFB)/280)*U(IP) 
            ENDIF

            CALL ALSBD(IELEM,1,Q2,IER,IELEM)
            IF(IER.NE.0) CALL XABORT('SNFT1P: SINGULAR MATRIX(3).')
            
            IF(IELEM.EQ.2)THEN
            CORNERQ(1) = Q2(1,IELEM+1) +
     >         Q2(2,IELEM+1)*(-SQRT(3.0)) 
            CORNERQ(2) = Q2(1,IELEM+1) + 
     >         Q2(2,IELEM+1)*(SQRT(3.0))
            ELSEIF(IELEM.EQ.3)THEN
            CORNERQ(1) = Q2(1,IELEM+1) + 
     >         Q2(2,IELEM+1)*(-SQRT(3.0)) + 
     >         Q2(3,IELEM+1)*(SQRT(5.0)) 
            CORNERQ(2) = Q2(1,IELEM+1) + 
     >         Q2(2,IELEM+1)*(SQRT(3.0)) + 
     >         Q2(3,IELEM+1)*(SQRT(5.0)) 
            ELSEIF(IELEM.EQ.4)THEN
            CORNERQ(1) = (Q2(01,IELEM+1) - 3**(0.5D0)*Q2(02,IELEM+1) 
     >         + 5**(0.5D0)*Q2(03,IELEM+1) - 7**(0.5D0)*Q2(04,IELEM+1)) 
            CORNERQ(2) = (Q2(01,IELEM+1) + 3**(0.5D0)*Q2(02,IELEM+1) 
     >         + 5**(0.5D0)*Q2(03,IELEM+1) + 7**(0.5D0)*Q2(04,IELEM+1)) 
            ENDIF
            AFB=CORNERQ(1)
  970    CONTINUE
*----
*  FORWARD SWEEP (FROM LEFT TO RIGHT).
*----
         IF(NCODE(1).EQ.4) THEN
            AFBB1=REAL(AFB)
            AFB1=AFBA1/(1.0+AFBA1-AFBB1)
            AFB=1.0
         ELSE
            AFB=ZCODE(1)*AFB
         ENDIF
         DO 990 I=1,NREG
            CORNERQ = 0.0
            IBM=MAT(I)
            
            DO IEL=1,IELEM
               Q(IEL)=0.0
               DO IL=0,NSCT-1
                  Q(IEL)=Q(IEL)+QEXT(IEL,IL+1,I)*PL(IL+1,NLF-IP+1)/2.0
               END DO 
            END DO
            
            IF(IELEM.EQ.2) THEN
            Q2(1,1) = U(NLF-IP+1)*(1.0/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/2.0) 
            Q2(2,1) = U(NLF-IP+1)*(-SQRT(3.0)/6.0) + 
     >         (TOTAL(IBM)*VOL(I))*(0.0) 
            Q2(1,2) = U(NLF-IP+1)*(SQRT(3.0)/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(0.0) 
            Q2(2,2) = U(NLF-IP+1)*(1.0/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/6.0) 
            Q2(1,3) = (Q(1)*(1.0/2.0))*VOL(I)
     >         + U(NLF-IP+1)*AFB*(1.0/2.0)
            Q2(2,3) = (Q(2)*(1.0/6.0))*VOL(I)
     >         + U(NLF-IP+1)*AFB*(-1.0*SQRT(3.0)/6.0)
            ELSE IF(IELEM.EQ.3) THEN
            Q2(1,1) = U(NLF-IP+1)*(1.0/6.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/2.0) 
            Q2(2,1) = U(NLF-IP+1)*(-SQRT(3.0)/6.0) + 
     >         (TOTAL(IBM)*VOL(I))*(0.0) 
            Q2(3,1) = U(NLF-IP+1)*(SQRT(5.0)/15.0) + 
     >         (TOTAL(IBM)*VOL(I))*(-SQRT(5.0)/15.0)

            Q2(1,2) = U(NLF-IP+1)*(5.0*SQRT(3.0)/6.0) + 
     >        (TOTAL(IBM)*VOL(I))*(0.0)
            Q2(2,2) = U(NLF-IP+1)*(1.0/2.0) + 
     >         (TOTAL(IBM)*VOL(I))*(1.0/6.0) 
            Q2(3,2) = U(NLF-IP+1)*(-SQRT(15.0)/5.0) + 
     >        (TOTAL(IBM)*VOL(I))*(0.0)

            Q2(1,3) = U(NLF-IP+1)*(5.0*SQRT(5.0)/30.0) + 
     >        (TOTAL(IBM)*VOL(I))*(-2.0*SQRT(5.0)/30.0)
            Q2(2,3) = U(NLF-IP+1)*(SQRT(15.0)/6.0) + 
     >        (TOTAL(IBM)*VOL(I))*(0.0)
            Q2(3,3) = U(NLF-IP+1)*(1.0/3.0) + 
     >        (TOTAL(IBM)*VOL(I))*(2.0/15.0)

            Q2(1,4) = (Q(1)*(90.0/180.0)
     >         + Q(3)*(-12.0*SQRT(5.0)/180.0))*VOL(I)
     >         + U(NLF-IP+1)*AFB*(1.0/6.0)
            Q2(2,4) = (Q(2)*(1.0/6.0))*VOL(I)
     >         + U(NLF-IP+1)*AFB*(-1.0*SQRT(3.0)/6.0)
            Q2(3,4) = (Q(1)*(-30.0*SQRT(5.0)/450.0) 
     >         + Q(3)*(12.0*(5.0)/450.0))*VOL(I)
     >         + U(NLF-IP+1)*AFB*(1.0*SQRT(5.0)/15.0)
            ELSE IF(IELEM.EQ.4) THEN
            Q2(1,1) = U(NLF-IP+1)/8 + (5*TOTAL(IBM)*VOL(I))/16 
            Q2(1,2) = (3**(0.5D0)*U(NLF-IP+1))/2 
            Q2(1,3) = (5**(0.5D0)*(10*U(NLF-IP+1) - 
     >         3*TOTAL(IBM)*VOL(I)))/80 
            Q2(1,4) = (7**(0.5D0)*U(NLF-IP+1))/8 

            Q2(2,1) = -(11*3**(0.5D0)*U(NLF-IP+1))/120 
            Q2(2,2) = (11*U(NLF-IP+1))/40 + 
     >         (17*TOTAL(IBM)*VOL(I))/48 
            Q2(2,3) = (37*15**(0.5D0)*U(NLF-IP+1))/60 
            Q2(2,4) = (3**(0.5D0)*7**(0.5D0)*(22*U(NLF-IP+1) - 
     >         9*TOTAL(IBM)*VOL(I)))/240 

            Q2(3,1) = (3*5**(0.5D0)*(2*U(NLF-IP+1) - 
     >         TOTAL(IBM)*VOL(I)))/80 
            Q2(3,2) = -(3*15**(0.5D0)*U(NLF-IP+1))/20 
            Q2(3,3) = (3*U(NLF-IP+1))/8 + (9*TOTAL(IBM)*VOL(I))/80 
            Q2(3,4) = (3*35**(0.5D0)*U(NLF-IP+1))/40 

            Q2(4,1) = -(9*7**(0.5D0)*U(NLF-IP+1))/280 
            Q2(4,2) = (3*3**(0.5D0)*7**(0.5D0)*(6*U(NLF-IP+1) - 
     >         7*TOTAL(IBM)*VOL(I)))/560 
            Q2(4,3) = -(9*35**(0.5D0)*U(NLF-IP+1))/35 
            Q2(4,4) = (9*U(NLF-IP+1))/40 + 
     >         (81*TOTAL(IBM)*VOL(I))/560 

            Q2(1,5) = (((50*Q(01) - 
     >         6*5**(0.5D0)*Q(03)))/160)*VOL(I) 
            Q2(2,5) = (((85*Q(02) - 
     >         9*21**(0.5D0)*Q(04)))/240)*VOL(I) 
            Q2(3,5) = ((3*(3*Q(03) - 
     >         5**(0.5D0)*Q(01)))/80)*VOL(I) 
            Q2(4,5) = ((3*(27*Q(04) - 
     >         7*21**(0.5D0)*Q(02)))/560)*VOL(I) 

            Q2(1,5) = Q2(1,5) + (AFB/8)*U(NLF-IP+1) 
            Q2(2,5) = Q2(2,5) + (-(11*3**(0.5D0)*AFB)/120)*U(NLF-IP+1) 
            Q2(3,5) = Q2(3,5) + ((3*5**(0.5D0)*AFB)/40)*U(NLF-IP+1) 
            Q2(4,5) = Q2(4,5) + (-(9*7**(0.5D0)*AFB)/280)*U(NLF-IP+1)
            ENDIF
            
            CALL ALSBD(IELEM,1,Q2,IER,IELEM)
            IF(IER.NE.0) CALL XABORT('SNFT1P: SINGULAR MATRIX(4).')
            
            IF(IELEM.EQ.2)THEN
            CORNERQ(1) = Q2(1,IELEM+1) +
     >         Q2(2,IELEM+1)*(-SQRT(3.0)) 
            CORNERQ(2) = Q2(1,IELEM+1) + 
     >         Q2(2,IELEM+1)*(SQRT(3.0))
            ELSEIF(IELEM.EQ.3)THEN
            CORNERQ(1) = Q2(1,IELEM+1) + 
     >         Q2(2,IELEM+1)*(-SQRT(3.0)) + 
     >         Q2(3,IELEM+1)*(SQRT(5.0)) 
            CORNERQ(2) = Q2(1,IELEM+1) + 
     >         Q2(2,IELEM+1)*(SQRT(3.0)) + 
     >         Q2(3,IELEM+1)*(SQRT(5.0)) 
            ELSEIF(IELEM.EQ.4)THEN
            CORNERQ(1) = (Q2(01,IELEM+1) - 3**(0.5D0)*Q2(02,IELEM+1) 
     >         + 5**(0.5D0)*Q2(03,IELEM+1) - 7**(0.5D0)*Q2(04,IELEM+1)) 
            CORNERQ(2) = (Q2(01,IELEM+1) + 3**(0.5D0)*Q2(02,IELEM+1) 
     >         + 5**(0.5D0)*Q2(03,IELEM+1) + 7**(0.5D0)*Q2(04,IELEM+1)) 
            ENDIF
            AFB=CORNERQ(2)
  990    CONTINUE
         IF(NCODE(1).EQ.4) THEN
            AFBB2=REAL(AFB)
            AFB2=AFBA2/(1.0+AFBA2-AFBB2)
         ELSE
            AFBB=REAL(AFB)
            AFB=ZCODE(2)*AFBA/(1.0+ZCODE(2)*(AFBA-AFBB))
         ENDIF
      ENDIF
*----
*  BACKWARD SWEEP (FROM RIGHT TO LEFT).
*----
      IF(NCODE(1).EQ.4) AFB=AFB1
      DO 1020 I=NREG,1,-1
         CORNERQ = 0.0
         IBM=MAT(I)
         
         DO IEL=1,IELEM
            Q(IEL)=0.0
            DO IL=0,NSCT-1
               Q(IEL)=Q(IEL)+QEXT(IEL,IL+1,I)*PL(IL+1,IP)/2.0
            END DO 
         END DO
         
         IF(IELEM.EQ.2) THEN
         Q2(1,1) = U(IP)*(-1.0/2.0) + 
     >      (TOTAL(IBM)*VOL(I))*(1.0/2.0) 
         Q2(2,1) = U(IP)*(-SQRT(3.0D0)/6.0) + 
     >      (TOTAL(IBM)*VOL(I))*(0.0) 
         Q2(1,2) = U(IP)*(SQRT(3.0D0)/2.0) + 
     >      (TOTAL(IBM)*VOL(I))*(0.0) 
         Q2(2,2) = U(IP)*(-1.0/2.0) + 
     >      (TOTAL(IBM)*VOL(I))*(1.0/6.0) 
         Q2(1,3) = (Q(1)*(1.0/2.0))*VOL(I)
     >      + U(IP)*AFB*(-1.0/2.0)
         Q2(2,3) = (Q(2)*(1.0/6.0))*VOL(I)
     >      + U(IP)*AFB*(-1.0*SQRT(3.0D0)/6.0)
         ELSE IF(IELEM.EQ.3) THEN
         Q2(1,1) = U(IP)*(-1.0/6.0) + 
     >      (TOTAL(IBM)*VOL(I))*(1.0/2.0) 
         Q2(2,1) = U(IP)*(-SQRT(3.0)/6.0) + 
     >      (TOTAL(IBM)*VOL(I))*(0.0) 
         Q2(3,1) = U(IP)*(-SQRT(5.0)/15.0) + 
     >      (TOTAL(IBM)*VOL(I))*(-SQRT(5.0)/15.0)

         Q2(1,2) = U(IP)*(5.0*SQRT(3.0)/6.0) + 
     >     (TOTAL(IBM)*VOL(I))*(0.0)
         Q2(2,2) = U(IP)*(-1.0/2.0) + 
     >      (TOTAL(IBM)*VOL(I))*(1.0/6.0) 
         Q2(3,2) = U(IP)*(-SQRT(15.0)/5.0) + 
     >     (TOTAL(IBM)*VOL(I))*(0.0)

         Q2(1,3) = U(IP)*(-5.0*SQRT(5.0)/30.0) + 
     >     (TOTAL(IBM)*VOL(I))*(-2.0*SQRT(5.0)/30.0)
         Q2(2,3) = U(IP)*(SQRT(15.0)/6.0) + 
     >     (TOTAL(IBM)*VOL(I))*(0.0)
         Q2(3,3) = U(IP)*(-1.0/3.0) + 
     >     (TOTAL(IBM)*VOL(I))*(2.0/15.0)

         Q2(1,4) = (Q(1)*(90.0/180.0)
     >      + Q(3)*(-12.0*SQRT(5.0)/180.0))*VOL(I)
     >      + U(IP)*AFB*(-1.0/6.0)
         Q2(2,4) = (Q(2)*(1.0/6.0))*VOL(I)
     >      + U(IP)*AFB*(-1.0*SQRT(3.0)/6.0)
         Q2(3,4) = (Q(1)*(-30.0*SQRT(5.0)/450.0) 
     >      + Q(3)*(12.0*(5.0)/450.0))*VOL(I)
     >      + U(IP)*AFB*(-1.0*SQRT(5.0)/15.0)
         ELSE IF(IELEM.EQ.4) THEN
         Q2(1,1) = (5*TOTAL(IBM)*VOL(I))/16 - U(IP)/8 
         Q2(1,2) = (3**(0.5D0)*U(IP))/2 
         Q2(1,3) = -(5**(0.5D0)*(10*U(IP) + 
     >      3*TOTAL(IBM)*VOL(I)))/80
         Q2(1,4) = (7**(0.5D0)*U(IP))/8 
            
         Q2(2,1) = -(11*3**(0.5D0)*U(IP))/120 
         Q2(2,2) = (17*TOTAL(IBM)*VOL(I))/48 - (11*U(IP))/40 
         Q2(2,3) = (37*15**(0.5D0)*U(IP))/60 
         Q2(2,4) = -(3**(0.5D0)*7**(0.5D0)*(22*U(IP) + 
     >      9*TOTAL(IBM)*VOL(I)))/240 
            
         Q2(3,1) = -(3*5**(0.5D0)*(2*U(IP) + 
     >      TOTAL(IBM)*VOL(I)))/80 
         Q2(3,2) = -(3*15**(0.5D0)*U(IP))/20 
         Q2(3,3) = (9*TOTAL(IBM)*VOL(I))/80 - (3*U(IP))/8 
         Q2(3,4) = (3*35**(0.5D0)*U(IP))/40 
            
         Q2(4,1) = -(9*7**(0.5D0)*U(IP))/280 
         Q2(4,2) = -(3*3**(0.5D0)*7**(0.5D0)*(6*U(IP) + 
     >      7*TOTAL(IBM)*VOL(I)))/560 
         Q2(4,3) = -(9*35**(0.5D0)*U(IP))/35 
         Q2(4,4) = (81*TOTAL(IBM)*VOL(I))/560 - (9*U(IP))/40 
            
         Q2(1,5) = (((50*Q(01) - 
     >      6*5**(0.5D0)*Q(03)))/160)*VOL(I) 
         Q2(2,5) = (((85*Q(02) - 
     >      9*21**(0.5D0)*Q(04)))/240)*VOL(I) 
         Q2(3,5) = ((3*(3*Q(03) - 
     >      5**(0.5D0)*Q(01)))/80)*VOL(I) 
         Q2(4,5) = ((3*(27*Q(04) - 
     >      7*21**(0.5D0)*Q(02)))/560)*VOL(I) 
            
         Q2(1,5) = Q2(1,5) + (-AFB/8)*U(IP) 
         Q2(2,5) = Q2(2,5) + (-(11*3**(0.5D0)*AFB)/120)*U(IP) 
         Q2(3,5) = Q2(3,5) + (-(3*5**(0.5D0)*AFB)/40)*U(IP) 
         Q2(4,5) = Q2(4,5) + (-(9*7**(0.5D0)*AFB)/280)*U(IP) 
         ENDIF

         CALL ALSBD(IELEM,1,Q2,IER,IELEM)
         IF(IER.NE.0) CALL XABORT('SNFT1P: SINGULAR MATRIX(5).')
         
         IF(IELEM.EQ.2)THEN
         CORNERQ(1) = Q2(1,IELEM+1) +
     >      Q2(2,IELEM+1)*(-SQRT(3.0)) 
         CORNERQ(2) = Q2(1,IELEM+1) + 
     >      Q2(2,IELEM+1)*(SQRT(3.0))
         ELSEIF(IELEM.EQ.3)THEN
         CORNERQ(1) = Q2(1,IELEM+1) + 
     >      Q2(2,IELEM+1)*(-SQRT(3.0)) + 
     >      Q2(3,IELEM+1)*(SQRT(5.0)) 
         CORNERQ(2) = Q2(1,IELEM+1) + 
     >      Q2(2,IELEM+1)*(SQRT(3.0)) + 
     >      Q2(3,IELEM+1)*(SQRT(5.0)) 
         ELSEIF(IELEM.EQ.4)THEN
         CORNERQ(1) = (Q2(01,IELEM+1) - 3**(0.5D0)*Q2(02,IELEM+1) 
     >      + 5**(0.5D0)*Q2(03,IELEM+1) - 7**(0.5D0)*Q2(04,IELEM+1)) 
         CORNERQ(2) = (Q2(01,IELEM+1) + 3**(0.5D0)*Q2(02,IELEM+1) 
     >      + 5**(0.5D0)*Q2(03,IELEM+1) + 7**(0.5D0)*Q2(04,IELEM+1)) 
         ENDIF
         AFB=CORNERQ(1)
         
         DO K=1,NSCT
            DO IEL=1,IELEM
               FLUX(IEL,K,I) = FLUX(IEL,K,I) + 
     1            W(IP)*REAL(Q2(IEL,IELEM+1))*PL(K,IP)
            END DO 
         END DO
 1020 CONTINUE
      CURR(1)=CURR(1)+W(IP)*U(IP)*AFB
*----
*  FORWARD SWEEP (FROM LEFT TO RIGHT).
*----
      AFB=ZCODE(1)*AFB
      IF(NCODE(1).EQ.4) AFB=AFB2
      DO 1050 I=1,NREG
         CORNERQ = 0.0
         IBM=MAT(I)
         
         DO IEL=1,IELEM
            Q(IEL)=0.0
            DO IL=0,NSCT-1
               Q(IEL)=Q(IEL)+QEXT(IEL,IL+1,I)*PL(IL+1,NLF-IP+1)/2.0
            END DO 
         END DO
         
         IF(IELEM.EQ.2) THEN
         Q2(1,1) = U(NLF-IP+1)*(1.0/2.0) + 
     >      (TOTAL(IBM)*VOL(I))*(1.0/2.0) 
         Q2(2,1) = U(NLF-IP+1)*(-SQRT(3.0)/6.0) + 
     >      (TOTAL(IBM)*VOL(I))*(0.0) 
         Q2(1,2) = U(NLF-IP+1)*(SQRT(3.0)/2.0) + 
     >      (TOTAL(IBM)*VOL(I))*(0.0) 
         Q2(2,2) = U(NLF-IP+1)*(1.0/2.0) + 
     >      (TOTAL(IBM)*VOL(I))*(1.0/6.0) 
         Q2(1,3) = (Q(1)*(1.0/2.0))*VOL(I)
     >      + U(NLF-IP+1)*AFB*(1.0/2.0)
         Q2(2,3) = (Q(2)*(1.0/6.0))*VOL(I)
     >      + U(NLF-IP+1)*AFB*(-1.0*SQRT(3.0)/6.0)
         ELSE IF(IELEM.EQ.3) THEN
         Q2(1,1) = U(NLF-IP+1)*(1.0/6.0) + 
     >      (TOTAL(IBM)*VOL(I))*(1.0/2.0) 
         Q2(2,1) = U(NLF-IP+1)*(-SQRT(3.0)/6.0) + 
     >      (TOTAL(IBM)*VOL(I))*(0.0) 
         Q2(3,1) = U(NLF-IP+1)*(SQRT(5.0)/15.0) + 
     >      (TOTAL(IBM)*VOL(I))*(-SQRT(5.0)/15.0)

         Q2(1,2) = U(NLF-IP+1)*(5.0*SQRT(3.0)/6.0) + 
     >     (TOTAL(IBM)*VOL(I))*(0.0)
         Q2(2,2) = U(NLF-IP+1)*(1.0/2.0) + 
     >      (TOTAL(IBM)*VOL(I))*(1.0/6.0) 
         Q2(3,2) = U(NLF-IP+1)*(-SQRT(15.0)/5.0) + 
     >     (TOTAL(IBM)*VOL(I))*(0.0)

         Q2(1,3) = U(NLF-IP+1)*(5.0*SQRT(5.0)/30.0) + 
     >     (TOTAL(IBM)*VOL(I))*(-2.0*SQRT(5.0)/30.0)
         Q2(2,3) = U(NLF-IP+1)*(SQRT(15.0)/6.0) + 
     >     (TOTAL(IBM)*VOL(I))*(0.0)
         Q2(3,3) = U(NLF-IP+1)*(1.0/3.0) + 
     >     (TOTAL(IBM)*VOL(I))*(2.0/15.0)

         Q2(1,4) = (Q(1)*(90.0/180.0)
     >      + Q(3)*(-12.0*SQRT(5.0)/180.0))*VOL(I)
     >      + U(NLF-IP+1)*AFB*(1.0/6.0)
            Q2(2,4) = (Q(2)*(1.0/6.0))*VOL(I)
     >      + U(NLF-IP+1)*AFB*(-1.0*SQRT(3.0)/6.0)
         Q2(3,4) = (Q(1)*(-30.0*SQRT(5.0)/450.0) 
     >         + Q(3)*(12.0*(5.0)/450.0))*VOL(I)
     >      + U(NLF-IP+1)*AFB*(1.0*SQRT(5.0)/15.0)
         ELSE IF(IELEM.EQ.4) THEN
         Q2(1,1) = U(NLF-IP+1)/8 + (5*TOTAL(IBM)*VOL(I))/16 
         Q2(1,2) = (3**(0.5D0)*U(NLF-IP+1))/2 
         Q2(1,3) = (5**(0.5D0)*(10*U(NLF-IP+1) - 
     >      3*TOTAL(IBM)*VOL(I)))/80 
         Q2(1,4) = (7**(0.5D0)*U(NLF-IP+1))/8 

         Q2(2,1) = -(11*3**(0.5D0)*U(NLF-IP+1))/120 
         Q2(2,2) = (11*U(NLF-IP+1))/40 + 
     >      (17*TOTAL(IBM)*VOL(I))/48 
         Q2(2,3) = (37*15**(0.5D0)*U(NLF-IP+1))/60 
         Q2(2,4) = (3**(0.5D0)*7**(0.5D0)*(22*U(NLF-IP+1) - 
     >      9*TOTAL(IBM)*VOL(I)))/240 

         Q2(3,1) = (3*5**(0.5D0)*(2*U(NLF-IP+1) - 
     >      TOTAL(IBM)*VOL(I)))/80 
         Q2(3,2) = -(3*15**(0.5D0)*U(NLF-IP+1))/20 
         Q2(3,3) = (3*U(NLF-IP+1))/8 + (9*TOTAL(IBM)*VOL(I))/80 
         Q2(3,4) = (3*35**(0.5D0)*U(NLF-IP+1))/40 

         Q2(4,1) = -(9*7**(0.5D0)*U(NLF-IP+1))/280 
         Q2(4,2) = (3*3**(0.5D0)*7**(0.5D0)*(6*U(NLF-IP+1) - 
     >      7*TOTAL(IBM)*VOL(I)))/560 
         Q2(4,3) = -(9*35**(0.5D0)*U(NLF-IP+1))/35 
         Q2(4,4) = (9*U(NLF-IP+1))/40 + 
     >      (81*TOTAL(IBM)*VOL(I))/560 

         Q2(1,5) = (((50*Q(01) - 
     >      6*5**(0.5D0)*Q(03)))/160)*VOL(I) 
         Q2(2,5) = (((85*Q(02) - 
     >      9*21**(0.5D0)*Q(04)))/240)*VOL(I) 
         Q2(3,5) = ((3*(3*Q(03) - 
     >      5**(0.5D0)*Q(01)))/80)*VOL(I) 
         Q2(4,5) = ((3*(27*Q(04) - 
     >      7*21**(0.5D0)*Q(02)))/560)*VOL(I) 

         Q2(1,5) = Q2(1,5) + (AFB/8)*U(NLF-IP+1) 
         Q2(2,5) = Q2(2,5) + (-(11*3**(0.5D0)*AFB)/120)*U(NLF-IP+1) 
         Q2(3,5) = Q2(3,5) + ((3*5**(0.5D0)*AFB)/40)*U(NLF-IP+1) 
         Q2(4,5) = Q2(4,5) + (-(9*7**(0.5D0)*AFB)/280)*U(NLF-IP+1)
         ENDIF
         
         CALL ALSBD(IELEM,1,Q2,IER,IELEM)
         IF(IER.NE.0) CALL XABORT('SNFT1P: SINGULAR MATRIX(6).')
         
         IF(IELEM.EQ.2)THEN
         CORNERQ(1) = Q2(1,IELEM+1) +
     >      Q2(2,IELEM+1)*(-SQRT(3.0)) 
         CORNERQ(2) = Q2(1,IELEM+1) + 
     >      Q2(2,IELEM+1)*(SQRT(3.0))
         ELSEIF(IELEM.EQ.3)THEN
         CORNERQ(1) = Q2(1,IELEM+1) + 
     >      Q2(2,IELEM+1)*(-SQRT(3.0)) + 
     >      Q2(3,IELEM+1)*(SQRT(5.0)) 
         CORNERQ(2) = Q2(1,IELEM+1) + 
     >      Q2(2,IELEM+1)*(SQRT(3.0)) + 
     >      Q2(3,IELEM+1)*(SQRT(5.0)) 
         ELSEIF(IELEM.EQ.4)THEN
         CORNERQ(1) = (Q2(01,IELEM+1) - 3**(0.5D0)*Q2(02,IELEM+1) 
     >      + 5**(0.5D0)*Q2(03,IELEM+1) - 7**(0.5D0)*Q2(04,IELEM+1)) 
         CORNERQ(2) = (Q2(01,IELEM+1) + 3**(0.5D0)*Q2(02,IELEM+1) 
     >      + 5**(0.5D0)*Q2(03,IELEM+1) + 7**(0.5D0)*Q2(04,IELEM+1)) 
         ENDIF
         AFB=CORNERQ(2)
         
         DO K=1,NSCT
            DO IEL=1,IELEM
               FLUX(IEL,K,I) = FLUX(IEL,K,I) + 
     1            W(NLF-IP+1)*REAL(Q2(IEL,IELEM+1))*PL(K,NLF-IP+1)
            END DO
         END DO
 1050 CONTINUE
      CURR(2)=CURR(2)+W(NLF-IP+1)*U(NLF-IP+1)*AFB

      DEALLOCATE(CORNERQ)
      ENDIF
*----
*  END OF OUTER LOOP OVER MU LEVELS
*----
  200 CONTINUE
      DEALLOCATE(Q,Q2)
      RETURN
      END
